# SAPM Exporter

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [deprecated]: traces   |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Aexporter%2Fsapm%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Aexporter%2Fsapm) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Aexporter%2Fsapm%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Aexporter%2Fsapm) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@dmitryax](https://www.github.com/dmitryax), [@atoulme](https://www.github.com/atoulme) |

[deprecated]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#deprecated
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

### Deprecated

> The SAPM protocol is based on the Jaeger protocol, which is no longer the preferred approach of the community, with the advent of the OTLP protocol. The Jaeger exporters have been removed from the collector already. Jaeger itself is migrating to use the OTel data format internally.

> As a result, the SAPM exporter is being deprecated in favor of the OTLPHTTP exporter. The following configuration can be used for the OTLPHTTP exporter

```yaml
exporters:
  otlphttp:
    traces_endpoint: "${SPLUNK_INGEST_URL}/v2/trace/otlp"
    headers:
        "X-SF-Token": "${SPLUNK_ACCESS_TOKEN}"
```
The SAPM exporter builds on the Jaeger proto and adds additional batching on top. This allows
the collector to export traces from multiples nodes/services in a single batch. The SAPM proto
and some useful related utilities is in [signalfx/sapm-proto](https://github.com/signalfx/sapm-proto/).

> Please review the Collector's [security
> documentation](https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/security-best-practices.md),
> which contains recommendations on securing sensitive information such as the
> API key required by this exporter.

## Configuration

The following configuration options are required:

- `access_token` (no default): AccessToken is the [authentication token provided by Splunk Observability Cloud](https://docs.splunk.com/observability/en/admin/authentication/authentication-tokens/manage-usage.html) or
another backend that supports the SAPM proto. The access token can be obtained from the
web app.
- `endpoint` (no default): This is the destination to where traces will be sent to in SAPM
format. It must be a full URL and include the scheme, port and path e.g,
<!-- markdown-link-check-disable-line -->https://ingest.us0.signalfx.com/v2/trace. This can be pointed to the SignalFx 
backend or to another Otel collector that has the SAPM receiver enabled.

The following configuration options can also be configured:

- `max_connections` (default = 100): MaxConnections is used to set a limit to the maximum
idle HTTP connection the exporter can keep open.
- `num_workers` (default = 8): NumWorkers is the number of workers that should be used to
export traces. Exporter can make as many requests in parallel as the number of workers. Note
that this will likely be removed in future in favour of processors handling parallel exporting.
- `access_token_passthrough`: (default = `true`) Whether to use `"com.splunk.signalfx.access_token"`
trace resource attribute, if any, as SFx access token.  In either case this attribute will be deleted
during final translation.  Intended to be used in tandem with identical configuration option for
[SAPM receiver](../../receiver/sapmreceiver/README.md) to preserve trace origin.
- `timeout` (default = 5s): Is the timeout for every attempt to send data to the backend.
- `log_detailed_response` (default = `false`): Option to log detailed response from Splunk APM.
In addition to setting this option to `true`, debug logging at the Collector level needs to be enabled.
- `compression`: Compression method to use for outgoing SAPM requests. Can be one of
  "gzip", "zstd" or be unspecified. If unspecified then "gzip" compression is used unless
  `disable_compression` option is set to true.
- `disable_compression` (default = `false`): If set to true the outgoing requests are not
  compressed and `compression` option is ignored.

In addition, this exporter offers queued retry which is enabled by default.
For more info, see the [exporterhelper configuration parameters](https://github.com/open-telemetry/opentelemetry-collector/blob/main/exporter/exporterhelper/README.md).

Example:

```yaml
exporters:
  sapm:
    access_token: YOUR_ACCESS_TOKEN
    access_token_passthrough: true
    endpoint: https://ingest.YOUR_SIGNALFX_REALM.signalfx.com/v2/trace
    max_connections: 100
    num_workers: 8
    log_detailed_response: true
```

The full list of settings exposed for this exporter are documented in [config.go](./config.go)
with detailed sample configurations in [testdata/config.yaml](./testdata/config.yaml).

This exporter also offers [proxy support](https://github.com/open-telemetry/opentelemetry-collector/tree/main/exporter#proxy-support).
