# Coralogix Processor

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [development]: traces   |
| Distributions | [] |
| Warnings      | [Statefulness](#warnings) |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Aprocessor%2Fcoralogix%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Aprocessor%2Fcoralogix) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Aprocessor%2Fcoralogix%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Aprocessor%2Fcoralogix) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@crobert-1](https://www.github.com/crobert-1), [@povilasv](https://www.github.com/povilasv) |

[development]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#development
<!-- end autogenerated section -->

## Description

The Coralogix processor adds attributes to spans that enable features in Coralogix.

## Features

### DB Statement Blueprints

This feature enables the processor to create blueprints from SQL queries, this means replacing any variables with `?`.
The blueprint is also hashed to be able to be used with the spanmetrics connector.
Long queries can be an issue when being stored in certain metric stores.
Blueprints alleviate this problem by using the hash as the identifying dimension on the metric, which enables
users to query metrics by blueprints.

The added attributes are `db.statement.blueprint` and `db.statement.blueprint.id`.

* `db.statement.blueprint` contains the blueprinted version of the statement, we require them to be sent to Coralogix to
  display your blueprinted statement
* `db.statement.blueprint.id` contains a hash of the statement, this way we can add it as a dimension in the spanmetrics
  connector and use it to query your blueprints.
* `sampling.priority` if enabled contains the value 100 for new blueprints, further explanation below.

#### Sampling

If sampling is enabled then it stores the found blueprints in an in-memory cache to be able to send only new blueprints
that haven't been seen yet.
This only adds an attribute to the span named `sampling.priority`, if the blueprint is new then the sampling priority
will be `100`.

Using this key it's possible to use either
the [Tail Sampler](https://github.com/open-telemetry/opentelemetry-collector-contrib/tree/main/processor/tailsamplingprocessor)
or
the [Probabilistic Sampler](https://github.com/open-telemetry/opentelemetry-collector-contrib/tree/main/processor/probabilisticsamplerprocessor)
to only send new blueprints to Coralogix.
If sampling is not enabled it won't cache anything and the `sampling.priority` attribute won't be added.

The cache is limited by the `max_cache_size_mib` configuration, if the cache is full it will remove the oldest entries
to make space for new ones.
The cache stores hashes of the queries, each hash is 8 bytes, so the number of maximum cache entries is calculated
by `max_cache_size_mib * 1024 * 1024 / 8`.

## Config

* `db_statement_blueprints`
    * `sampling`:
        * `enabled`: (default: `false`) If enabled, adds the attribute `sampling.priority` with a value of `100` to spans with new
          blueprints.
          Refer to the [Sampling section](#sampling) for more information.
        * `max_cache_size_mib` (default: `1024`) The size of the cache in mebibytes to store seen blueprints hashes.

### Basic Setup

This setup is without sampling meaning no `sampling.priority` attribute will be added to spans.
The cache will be disabled.

```yaml
processors:
  coralogix:
    db_statement_blueprints:
```

### With Sampling Config

This setup will enable the cache to store seen blueprints and add the `sampling.priority` attribute to spans with new
blueprints.

```yaml
processors:
  coralogix:
    db_statement_blueprints:
      sampling:
        enabled: true
        max_cache_size_mib: 1024 #1GiB
  ```