# MongoDB Atlas Receiver

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [beta]: metrics, logs   |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Areceiver%2Fmongodbatlas%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Areceiver%2Fmongodbatlas) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Areceiver%2Fmongodbatlas%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Areceiver%2Fmongodbatlas) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@schmikei](https://www.github.com/schmikei) \| Seeking more code owners! |

[beta]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#beta
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

Receives metrics from [MongoDB Atlas](https://www.mongodb.com/cloud/atlas) 
via their [monitoring APIs](https://docs.atlas.mongodb.com/reference/api/monitoring-and-logs/),
as well as alerts via a configured [webhook](https://www.mongodb.com/docs/atlas/tutorial/third-party-service-integrations/)
and events from [events APIs](https://www.mongodb.com/docs/atlas/reference/api/events/).

## Getting Started

The MongoDB Atlas receiver takes the following parameters. `public_key` and 
`private_key` are the only two required values to receive metrics and logs and are obtained via the
"API Keys" tab of the MongoDB Atlas Project Access Manager. In the example
below both values are being pulled from the environment.

In order to collect logs, at least one project must be specified. By default, logs for all clusters within a project will be collected. Clusters can be limited using either the `include_clusters` or `exclude_clusters` setting.

In order to collect project events, the requesting API key needs the appropriate permission which at minimum is the `Project Read Only` role. Project events are specific to a single project.

In order to collect organization events, the requesting API key needs the appropriate permission which at minimum is the `Organization Member` role. Organization events are collected across all the projects hosted on Atlas within the organization. These events are not associated with a project.

In order to collect access logs, the requesting API key needs the appropriate permission which requires either the `Project Owner` or `Organization Owner` role. Access logs are specific to each cluster.

MongoDB Atlas [Documentation](https://www.mongodb.com/docs/atlas/reference/api/logs/#logs) recommends a polling interval of 5 minutes.

- `public_key` (required for metrics, logs, or alerts in `poll` mode)
- `private_key` (required for metrics, logs, or alerts in `poll` mode)
- `granularity` (default `PT1M` - See [MongoDB Atlas Documentation](https://docs.atlas.mongodb.com/reference/api/process-measurements/))
- `collection_interval` (default `3m`) This receiver collects metrics on an interval. Valid time units are `ns`, `us` (or `µs`), `ms`, `s`, `m`, `h`.
- `storage` (optional) The component ID of a storage extension which can be used when polling for `alerts` or `events` . The storage extension prevents duplication of data after a collector restart by remembering which data were previously collected.
- `projects` (optional for metrics) a slice of projects this receiver collects metrics from instead of all projects in an organization
  - `name` Name of the project to discover metrics from
  - `include_clusters` (default empty, exclusive with `exclude_clusters`)
  - `exclude_clusters` (default empty, exclusive with `include_clusters`)
    - If both `include_clusters` and `exclude_clusters` are empty, then all clusters in the project will be included
- `retry_on_failure`
  - `enabled` (default true)
  - `initial_interval` (default 5s)
  - `max_interval` (default 30s)
  - `max_elapsed_time` (default 5m)
- `alerts`
  - `enabled` (default false)
  - `mode` (default `listen`. Options are `poll` or `listen`)
  - `secret` (required if using `listen` mode)
  - `endpoint` (required if using `listen` mode)
  - `poll_interval` (default `5m`, only relevant using `poll` mode)
  - `page_size` (default `100`)
    - When in `poll` mode, this is the number of alerts that will be processed per request to the MongoDB Atlas API.
  - `max_pages` (default `10`)
    - When in `poll` mode, this will limit how many pages of alerts the receiver will request for each project.
  - `projects` (required if using `poll` mode)
    - `name` (required if using `poll mode`)
    - `include_clusters` (default empty, exclusive with `exclude_clusters`)
    - `exclude_clusters` (default empty, exclusive with `include_clusters`)
      - If both `include_clusters` and `exclude_clusters` are empty, then all clusters in the project will be included
  - `tls` (relevant only for `listen` mode)
    - `key_file`
    - `cert_file`
- `logs`
  - `enabled` (default false)
  - `projects` (required if enabled)
    - `name` (required if enabled)
    - `collect_host_logs` (default true)
    - `collect_audit_logs` (default false)
      - Audit logging must be [enabled for your MongoDB Atlas project](https://www.mongodb.com/docs/atlas/database-auditing/) in order to scrape audit logs.
    - `access_logs`
      - `enabled` (default true, if the `access_logs` parameter is defined)
      - `auth_result`
        - If specified, will limit the access logs queried to successful accesses (true) or failed accesses (false). If not specified, all will be collected
      - `page_size` (default `20000`)
        - This is the number of access logs that will be processed per request to the MongoDB Atlas API. The maximum value is 20000.
      - `max_pages` (default `10`)
        - This will limit how many pages of access logs the receiver will request from the MongoDB Atlas API for a project.
      - `poll_interval` (default `5m`)
        - This will define how frequently the MongoDB Atlas API is queried for Access Logs for the given project.
    - `include_clusters` (default empty)
    - `exclude_clusters` (default empty)
- `events`
  - `projects`
    - `name` Name of the Project to discover events from.
  - `organizations`
    - `id` ID of the Organization to discover events from.
  - `poll_interval` (default `1m`)
    - How often the receiver will poll the Events API for new events.
  - `page_size` (default `100`)
    - This is the number of events that will be processed per request to the MongoDB Atlas API.
  - `max_pages` (default `25`)
    - This will limit how many pages of events the receiver will request from the MongoDB Atlas API for each project.
  - `types` (defaults to all types of events)
    - This is a list of [event types](https://www.mongodb.com/docs/atlas/reference/api/events-orgs-get-all/#event-type-values) that the receiver will request from the API. If specified, the receiver will collect only the indicated types of events.

Examples:

Receive metrics:

```yaml
receivers:
  mongodbatlas:
    public_key: ${env:MONGODB_ATLAS_PUBLIC_KEY}
    private_key: ${env:MONGODB_ATLAS_PRIVATE_KEY}
```

Listen for alerts (default mode):

```yaml
receivers:
  mongodbatlas:
    alerts:
      enabled: true
      secret: "some_secret"
      endpoint: "0.0.0.0:7706"
```

Poll alerts from API:

```yaml
receivers:
  mongodbatlas:
    public_key: <redacted>
    private_key: <redacted>
    alerts:
      enabled: true
      mode: poll
      projects:
      - name: Project 0
        include_clusters: [Cluster0]
      poll_interval: 1m
    # use of a storage extension is recommended to reduce chance of duplicated alerts
    storage: file_storage
```

Receive logs:

```yaml
receivers:
  mongodbatlas:
    logs:
      enabled: true
      projects: 
        - name: "project 1"
          collect_audit_logs: true
          collect_host_logs: true
```

Receive events:

```yaml
receivers:
  mongodbatlas:
    events:
      projects:
        - name: "project 1"
      organizations:
        - id: "5b478b3afc4625789ce616a3"
      poll_interval: 1m
      page_size: 100
      max_pages: 25
    # use of a storage extension is recommended to reduce chance of duplicated events
    storage: file_storage
```

Poll Access Logs from API:

```yaml
receivers:
  mongodbatlas:
    public_key: <redacted>
    private_key: <redacted>
    logs:
      enabled: true
      projects:
      - name: Project 0
        include_clusters: [Cluster0]
        access_logs:
          page_size: 20000
          max_pages: 10
          poll_interval: 5m
    # use of a storage extension is recommended to reduce chance of duplicated access logs
    storage: file_storage
```
