# Pulsar Receiver

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [alpha]: metrics, traces, logs   |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Areceiver%2Fpulsar%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Areceiver%2Fpulsar) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Areceiver%2Fpulsar%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Areceiver%2Fpulsar) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@dmitryax](https://www.github.com/dmitryax), [@dao-jun](https://www.github.com/dao-jun) |

[alpha]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#alpha
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

Pulsar receiver receives logs, metrics, and traces from [Apache Pulsar](https://pulsar.apache.org/).

## Getting Started

The following settings can be optionally configured:
- `endpoint` (default = pulsar://localhost:6650): The url of pulsar cluster.
- `topic` (default = otlp_spans for traces, otlp_metrics for metrics, otlp_logs for logs): The name of the pulsar topic to consume from.
- `encoding` (default = otlp_proto): The encoding of the payload sent to pulsar. Available encodings:
    - `otlp_proto`: the payload is deserialized to `ExportTraceServiceRequest`.
    - `jaeger_proto`: the payload is deserialized to a single Jaeger proto `Span`.
    - `jaeger_json`: the payload is deserialized to a single Jaeger JSON Span using `jsonpb`.
    - `zipkin_proto`: the payload is deserialized into a list of Zipkin proto spans.
    - `zipkin_json`: the payload is deserialized into a list of Zipkin V2 JSON spans.
    - `zipkin_thrift`: the payload is deserialized into a list of Zipkin Thrift spans.
- `consumer_name`: specifies the consumer name.
- `auth`
  - `tls`
    - `cert_file`:
    - `key_file`:
  - `token`
    - `token`
  - `oauth2`
    - `issuer_url`:
    - `client_id`:
    - `audience`: 
  - `athenz`
    - `provider_domain`:
    - `tenant_domain`:
    - `tenant_service`:
    - `private_key`:
    - `key_id`:
    - `principal_header`:
    - `zts_url`:
- `subscription` (default = otlp_subscription): the subscription name of consumer.
- `tls_trust_certs_file_path`: path to the CA cert. For a client this verifies the server certificate. Should
  only be used if `insecure` is set to true.
- `tls_allow_insecure_connection`: configure whether the Pulsar client accept untrusted TLS certificate from broker (default: false)


Example configuration:
```yaml
receivers:
  pulsar:
    endpoint: pulsar://localhost:6650
    topic: otlp-spans
    subscription: otlp_spans_sub
    consumer_name: otlp_spans_sub_1
    encoding: otlp_proto
    auth:
      tls:
        cert_file: cert.pem
        key_file: key.pem
    tls_allow_insecure_connection: false
    tls_trust_certs_file_path: ca.pem
```

