# SignalFx Receiver

<!-- status autogenerated section -->
| Status        |           |
| ------------- |-----------|
| Stability     | [beta]: metrics, logs   |
| Distributions | [contrib] |
| Issues        | [![Open issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aopen%20label%3Areceiver%2Fsignalfx%20&label=open&color=orange&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aopen+is%3Aissue+label%3Areceiver%2Fsignalfx) [![Closed issues](https://img.shields.io/github/issues-search/open-telemetry/opentelemetry-collector-contrib?query=is%3Aissue%20is%3Aclosed%20label%3Areceiver%2Fsignalfx%20&label=closed&color=blue&logo=opentelemetry)](https://github.com/open-telemetry/opentelemetry-collector-contrib/issues?q=is%3Aclosed+is%3Aissue+label%3Areceiver%2Fsignalfx) |
| [Code Owners](https://github.com/open-telemetry/opentelemetry-collector-contrib/blob/main/CONTRIBUTING.md#becoming-a-code-owner)    | [@dmitryax](https://www.github.com/dmitryax) |

[beta]: https://github.com/open-telemetry/opentelemetry-collector/blob/main/docs/component-stability.md#beta
[contrib]: https://github.com/open-telemetry/opentelemetry-collector-releases/tree/main/distributions/otelcol-contrib
<!-- end autogenerated section -->

The SignalFx receiver accepts:

- Metrics in the [SignalFx proto
format](https://github.com/signalfx/com_signalfx_metrics_protobuf).
- Events (Logs) in the [SignalFx proto
format](https://github.com/signalfx/com_signalfx_metrics_protobuf/blob/master/proto/signalfx_metrics.proto#L137).
More information about sending custom events can be found in the [SignalFx
Developers
Guide](https://dev.splunk.com/observability/reference/api/ingest_data/latest).

## Configuration

The following settings are required:

- `endpoint` (default = `localhost:9943`): Address and port that the SignalFx
  receiver should bind to.
  See our [security best practices doc](https://opentelemetry.io/docs/security/config-best-practices/#protect-against-denial-of-service-attacks) to understand how to set the endpoint in different environments.

The following settings are optional:

- `access_token_passthrough`: (default = `false`) Whether to preserve incoming
  access token (`X-Sf-Token` header value) as
  `"com.splunk.signalfx.access_token"` metric resource attribute.  Should only be
  used in tandem with identical configuration option for [SignalFx
  exporter](../../exporter/signalfxexporter/README.md) to preserve datapoint
  origin.  Usage of any other exporter in a metric pipeline with this configuration
  option enabled will reveal all organization access tokens contained in this attribute.
- `tls_settings` (no default): This is an optional object used to specify if
  TLS should be used for incoming connections. Both `key_file` and `cert_file`
  are required to support incoming TLS connections.
    - `cert_file`: Specifies the certificate file to use for TLS connection.
    - `key_file`: Specifies the key file to use for TLS connection.

Example:

```yaml
receivers:
  signalfx:
  signalfx/advanced:
    access_token_passthrough: true
    tls:
      cert_file: /test.crt
      key_file: /test.key
```

The full list of settings exposed for this receiver are documented in [config.go](./config.go)
with detailed sample configurations in [testdata/config.yaml](./testdata/config.yaml).

> :warning: When enabling the SignalFx receiver or exporter, configure both the `metrics` and `logs` pipelines.

```yaml
service:
  pipelines:
    metrics:
      receivers: [signalfx]
      processors: [memory_limiter, batch]
      exporters: [signalfx]
    logs:
      receivers: [signalfx]
      processors: [memory_limiter, batch]
      exporters: [signalfx]
```
